%% Reproduce Figure 10 (discussion on noise levels) in the following paper:
%% Hengfa Lu et al., Low rank enhanced matrix recovery of hybrid time and frequency data in fast magnetic resonance spectroscopy, IEEE Transactions on Biomedical Engineering, 10.1109/TBME.2017.2719709, 2017.
%% Created by Hengfa Lu
%% Corresponding Author: Xiaobo Qu
%% Homepage: http://csrc.xmu.edu.cn
%% Email: quxiaobo <|at|> xmu.edu.cn
%% Affiliations: Computational Sensing Group (CSG), Department of Electronic Science, Xiamen University, Xiamen 361005, China
%% All rights are reserved by CSG. 

clc;
clear all;
close all;

%% add path
currentFolder = pwd;
addpath(genpath(currentFolder));

%% generate synthetic data and mask
load SyntheticData_64x64_10Peaks
load pseudoPoissonGapMask_64x64_SR_020

[signal_row, signal_col] = size(FID);
hybridData = fft(FID,[],2);
normalizedFactor = max(abs(hybridData(:)));
hybridData = hybridData / normalizedFactor;

%% reconstruction
CS_Params.lambda = 1e3;
CS_Params.tol = 1e-4;

LRBHM_Params.tol = 5*1e-4;
LRBHM_Params.lambda = 1e3;

noiseVec = [0.005 0.05 0.1];
lambdaVec = [50 1e2 5*1e2 1e3 5*1e3 1e4 5*1e4 1e5 5*1e5 1e6];

CS_RLNE = zeros(size(noiseVec,2),size(lambdaVec,2));
LRBHM_RLNE = zeros(size(noiseVec,2),size(lambdaVec,2));

all_Noise = zeros(size(noiseVec,2),signal_row,signal_col);
all_CS_RecSpec = zeros(size(noiseVec,2),size(lambdaVec,2),signal_row,signal_col);
all_LRBHM_RecFID = zeros(size(noiseVec,2),size(lambdaVec,2),signal_row,signal_col);

%
for noiseID = 1:size(noiseVec,2)
    %% add noise
    noiseSTD = noiseVec(1,noiseID);
    noise = noiseSTD * ( randn( signal_row,signal_col ) + 1i*randn( signal_row, signal_col ) );
    hybridData_noisy = hybridData + noise;
    all_Noise(noiseID,:,:) = noise;
    
    %% down sampling
    zerosFillingSampledSignal = zeros(signal_row,signal_col);
    index = find(mask==1);
    zerosFillingSampledSignal(index) = hybridData_noisy(index);
    
    for LambdaID = 1:size(lambdaVec,2)
        fprintf('\n Noise level: %.3f, lambda: %d\n',noiseVec(1,noiseID),lambdaVec(1,LambdaID));
        % CS
        CS_Params.lambda = lambdaVec(1,LambdaID);
        CS_RecSpec = Solver_HybridSignalReconstruction_CS(zerosFillingSampledSignal,mask,CS_Params);
        
        all_CS_RecSpec(noiseID,LambdaID,:,:) = CS_RecSpec;
        CS_RecFID = ifft(CS_RecSpec,[],1);
        CS_RecFID = CS_RecFID * normalizedFactor;
        CS_RecFID = ifft(CS_RecFID,[],2);
        CS_RLNE(noiseID, LambdaID) = rlne(FID,CS_RecFID);
        
        % LRBHM
        LRBHM_Params.lambda = lambdaVec(1,LambdaID);
        LRBHM_RecHybridData = Solver_HybridSignalReconstruction_LRBHM_HTF(zerosFillingSampledSignal,mask,LRBHM_Params);
        
        LRBHM_RecHybridData = LRBHM_RecHybridData * normalizedFactor;
        LRBHM_RecFID = ifft(LRBHM_RecHybridData,[],2);
        all_LRBHM_RecFID(noiseID,LambdaID,:,:) = LRBHM_RecFID;
        LRBHM_RLNE(noiseID, LambdaID) = rlne(FID,LRBHM_RecFID);
    end
    
end

%% select the best lambda for CS and LRBHM-HTF and display results
% sigma = 0.005
[~,index] = min(CS_RLNE(1,:));
CS_spec_0005 = squeeze(all_CS_RecSpec(1,index,:,:));
[~,index] = min(LRBHM_RLNE(1,:));
LRBHM_spec_0005 = fft2(squeeze(all_LRBHM_RecFID(1,index,:,:)));

% sigma = 0.05
[~,index] = min(CS_RLNE(2,:));
CS_spec_005 = squeeze(all_CS_RecSpec(2,index,:,:));
[~,index] = min(LRBHM_RLNE(2,:));
LRBHM_spec_005 = fft2(squeeze(all_LRBHM_RecFID(2,index,:,:)));

% sigma = 0.1
[~,index] = min(CS_RLNE(3,:));
CS_spec_01 = squeeze(all_CS_RecSpec(3,index,:,:));
[~,index] = min(LRBHM_RLNE(3,:));
LRBHM_spec_01 = fft2(squeeze(all_LRBHM_RecFID(3,index,:,:)));

% display spectra
CTLevel = 10;
figure,
subplot(4,3,1); contour(abs(fft2(FID)),CTLevel); subplot(4,3,2); contour(abs(hybridData).',CTLevel); subplot(4,3,3); contour(mask);
subplot(4,3,4); contour(abs(fft(hybridData+squeeze(all_Noise(1,:,:)),[],1)),CTLevel);
subplot(4,3,5); contour(abs(fft(hybridData+squeeze(all_Noise(2,:,:)),[],1)),CTLevel);
subplot(4,3,6); contour(abs(fft(hybridData+squeeze(all_Noise(3,:,:)),[],1)),CTLevel);

subplot(4,3,7); contour(abs(CS_spec_0005),CTLevel);  subplot(4,3,8); contour(abs(CS_spec_005),CTLevel); subplot(4,3,9); contour(abs(CS_spec_01),CTLevel); 

subplot(4,3,10); contour(abs(LRBHM_spec_0005),CTLevel); subplot(4,3,11); contour(abs(LRBHM_spec_005),CTLevel); subplot(4,3,12); contour(abs(LRBHM_spec_01),CTLevel);


%% ----------------- The end of the file ----------------------------------