%% Reproduce Figure 11 (discussion on sampling patterns for STEU COSY) in the following paper:
%% Hengfa Lu et al., Low rank enhanced matrix recovery of hybrid time and frequency data in fast magnetic resonance spectroscopy, IEEE Transactions on Biomedical Engineering, 10.1109/TBME.2017.2719709, 2017.
%% Created by Hengfa Lu
%% Corresponding Author: Xiaobo Qu
%% Homepage: http://csrc.xmu.edu.cn
%% Email: quxiaobo <|at|> xmu.edu.cn
%% Affiliations: Computational Sensing Group (CSG), Department of Electronic Science, Xiamen University, Xiamen 361005, China
%% All rights are reserved by CSG. 

clc;
clear all;
close all;

%% add path
currentFolder = pwd;
addpath(genpath(currentFolder));

%% generate synthetic data and mask
load marrow_cosy;
load GradientSwitch_samplingPattern_DiffSR;

[signal_row, signal_col] = size(dataO);
hybridData = dataO;
normalizedFactor = max(abs(hybridData(:)));
hybridData = hybridData / normalizedFactor;

%% LRBHM reconstruction
CS_params.lambda = 1e5;
CS_params.Tol = 1e-4;

LRBHM_Params.tol = 5*1e-4;
LRBHM_Params.lambda = 1e7;

all_CS_RecFID = zeros(size(GradientSwitch_samplingPattern_DiffSR,3),signal_row,signal_col);
all_LRBHM_RecFID = zeros(size(GradientSwitch_samplingPattern_DiffSR,3),signal_row,signal_col);

for sampingRateID = 1:size(GradientSwitch_samplingPattern_DiffSR,3)
    
    mask = GradientSwitch_samplingPattern_DiffSR(:,:,sampingRateID);
    fprintf('Sampling rate: %.2f \n', sum(mask(:))./numel(mask(:)));
    
    %% down sampling    
    zerosFillingSampledSignal = zeros(signal_row,signal_col);
    index = find(mask==1);
    zerosFillingSampledSignal(index) = hybridData(index);
    
    %% CS reconstruction
    CS_RecSpectrum = Solver_HybridSignalReconstruction_CS(zerosFillingSampledSignal,mask,CS_params);

    CS_RecFID = ifft2(CS_RecSpectrum);
    all_CS_RecFID(sampingRateID,:,:) = CS_RecFID;
    
    %% LRBHM reconstruction  
    LRBHM_RecHybridData = Solver_HybridSignalReconstruction_LRBHM_HTF(zerosFillingSampledSignal,mask,LRBHM_Params);

    all_LRBHM_RecFID(sampingRateID,:,:) = ifft(LRBHM_RecHybridData,[],2);
    
end

%% display spectra
N1 = signal_col;
N2 = 100;
fn1 = 1024;
fn2 = 1024;
CTlevel = 90;
win1Weight = 0.4;
win2Weight = 0.6;

SpectrumPostProcessing(ifft(circshift(fft(hybridData,[],1),[-2,5]),[],1),N1,N2,fn1,fn2,win1Weight,win2Weight,1,CTlevel); title('Ground Truth');

SpectrumPostProcessing(ifft(circshift(fft2(squeeze(all_CS_RecFID(1,:,:))),[-2,5]),[],1),N1,N2,fn1,fn2,win1Weight,win2Weight,1,CTlevel); title('CS - 20%');
SpectrumPostProcessing(ifft(circshift(fft2(squeeze(all_CS_RecFID(2,:,:))),[-2,5]),[],1),N1,N2,fn1,fn2,win1Weight,win2Weight,1,CTlevel); title('CS - 40%');
SpectrumPostProcessing(ifft(circshift(fft2(squeeze(all_CS_RecFID(3,:,:))),[-2,5]),[],1),N1,N2,fn1,fn2,win1Weight,win2Weight,1,CTlevel); title('CS - 50%');
SpectrumPostProcessing(ifft(circshift(fft2(squeeze(all_LRBHM_RecFID(1,:,:))),[-2,5]),[],1),N1,N2,fn1,fn2,win1Weight,win2Weight,1,CTlevel); title('LRBHM-HTF - 20%');
SpectrumPostProcessing(ifft(circshift(fft2(squeeze(all_LRBHM_RecFID(2,:,:))),[-2,5]),[],1),N1,N2,fn1,fn2,win1Weight,win2Weight,1,CTlevel); title('LRBHM-HTF - 40%');
SpectrumPostProcessing(ifft(circshift(fft2(squeeze(all_LRBHM_RecFID(3,:,:))),[-2,5]),[],1),N1,N2,fn1,fn2,win1Weight,win2Weight,1,CTlevel); title('LRBHM-HTF - 50%');


%% lines shape analysis
N1 = 100;
N2 = 100;

postPro_Ori_RecSpec = SpectrumPostProcessing(dataO,N1,N2,fn1,fn2,win1Weight,win2Weight,0,CTlevel);
postPro_CS_RecSpec  = SpectrumPostProcessing(fft(squeeze(all_CS_RecFID(3,:,:)),[],2),N1,N2,fn1,fn2,win1Weight,win2Weight,0,CTlevel);
postPro_LRBHM_RecSpec = SpectrumPostProcessing(fft(squeeze(all_LRBHM_RecFID(3,:,:)),[],2),N1,N2,fn1,fn2,win1Weight,win2Weight,0,CTlevel);

% low intensities peak
x_ppm = (4.05-(353-323)*0.005):0.005:((323-260)*0.005+4.05);

figure, plot(x_ppm, fliplr(abs(postPro_Ori_RecSpec(235,260:353))), 'LineWidth', 3, 'color', [0 0.45 0.74]);  hold on;
plot(x_ppm, fliplr(abs(postPro_CS_RecSpec(235,260:353))), 'LineWidth', 3, 'color', [0 0.5 0]); hold on;
plot(x_ppm, fliplr(abs(postPro_LRBHM_RecSpec(235,260:353))), 'LineWidth', 3, 'color', [0.85 0.33 0.1]);
xlim([min(x_ppm) max(x_ppm)]); ylim([0 0.04]); legend('Reference','CS','Proposed method','Location','NorthWest');

% moderate intensities peak
x_ppm = (2.02-(729-699)*0.005):0.005:((699-669)*0.005+2.02);

figure, plot(x_ppm,fliplr(abs(postPro_Ori_RecSpec(550,669:729))), 'LineWidth', 3, 'color', [0 0.45 0.74]); hold on;
plot(x_ppm,fliplr(abs(postPro_CS_RecSpec(550,669:729))), 'LineWidth', 3, 'color', [0 0.5 0]); hold on;
plot(x_ppm,fliplr(abs(postPro_LRBHM_RecSpec(550,669:729))), 'LineWidth', 3, 'color', [0.85 0.33 0.1]); 
xlim([min(x_ppm) max(x_ppm)]); ylim([0 0.2]);  legend('Reference','CS','Proposed method','Location','NorthWest');

% high intensities peak
x_ppm = (1.3-(970-896)*0.005):0.005:((896-810)*0.005+1.3);

figure, plot(x_ppm, fliplr(abs(postPro_Ori_RecSpec(711,810:970))), 'LineWidth', 3, 'color', [0 0.45 0.74]); hold on;
plot(x_ppm, fliplr(abs(postPro_CS_RecSpec(711,810:970))), 'LineWidth', 3, 'color', [0 0.5 0]); hold on;
plot(x_ppm, fliplr(abs(postPro_LRBHM_RecSpec(711,810:970))), 'LineWidth', 3, 'color', [0.85 0.33 0.1]);
xlim([min(x_ppm) max(x_ppm)]); ylim([0 1]); legend('Reference','CS','Proposed method','Location','NorthWest');

%% ----------------- The end of the file ----------------------------------