%% Reproduce Figure 12 (discussion on Lambda) in the following paper:
%% Hengfa Lu et al., Low rank enhanced matrix recovery of hybrid time and frequency data in fast magnetic resonance spectroscopy, IEEE Transactions on Biomedical Engineering, 10.1109/TBME.2017.2719709, 2017.
%% Created by Hengfa Lu
%% Corresponding Author: Xiaobo Qu
%% Homepage: http://csrc.xmu.edu.cn
%% Email: quxiaobo <|at|> xmu.edu.cn
%% Affiliations: Computational Sensing Group (CSG), Department of Electronic Science, Xiamen University, Xiamen 361005, China
%% All rights are reserved by CSG. 

clc;
clear all;
close all;
clear classes;

%% add path
currentFolder = pwd;
addpath(genpath(currentFolder));

%% generate synthetic data and mask
load SyntheticData_64x64_5Peaks_KnowParam
load pseudoPoissonGapMask_64x64_SR_050

[signal_row, signal_col] = size(FID);
hybridData = fft(FID,[],2);
normalizedFactor = max(abs(hybridData(:)));
hybridData = hybridData / normalizedFactor;

%% reconstruction
CS_Params.lambda = 1e3;
CS_Params.Tol = 1e-4;

LRBHM_Params.tol = 5*1e-4;
LRBHM_Params.lambda = 1e3;

noiseLevelsVec = [0.005 0.015 0.025 0.05];
lambdaVec = [10 50 1e2 2*1e2 5*1e2 1e3 2*1e3 5*1e3 1e4 5*1e4 1e5 5*1e5 1e6 1e7 1e8 1e9];

CS_RLNE = zeros(size(noiseLevelsVec,2),size(lambdaVec,2));
LRBHM_RLNE = zeros(size(noiseLevelsVec,2),size(lambdaVec,2));

allNoise = zeros(size(noiseLevelsVec,2),signal_row,signal_col);
all_CS_RecSpec = zeros(size(noiseLevelsVec,2),size(lambdaVec,2),signal_row,signal_col);
all_LRBHM_RecFID = zeros(size(noiseLevelsVec,2),size(lambdaVec,2),signal_row,signal_col);


%
for noiseLevelID = 1:size(noiseLevelsVec,2)
    %% add noise
    noiseSTD = noiseLevelsVec(1,noiseLevelID);
    noise = noiseSTD * ( randn( signal_row,signal_col ) + 1i*randn( signal_row, signal_col ) );
    hybridData_noisy = hybridData + noise;
    
    allNoise(noiseLevelID,:,:) = noise;
    
    %% down sampling
    zerosFillingSampledSignal = zeros(signal_row,signal_col);
    index = find(mask==1);
    zerosFillingSampledSignal(index) = hybridData_noisy(index);
    
    for lambdaID = 1:size(lambdaVec,2)
        fprintf('\n Noise level: %.3f, lambda: %d\n',noiseLevelsVec(1,noiseLevelID),lambdaVec(1,lambdaID));
        % CS
        CS_Params.lambda = lambdaVec(1,lambdaID);
        CS_RecSpectrum = Solver_HybridSignalReconstruction_CS(zerosFillingSampledSignal,mask,CS_Params);
        all_CS_RecSpec(noiseLevelID,lambdaID,:,:) = CS_RecSpectrum;
        
        CS_RecFID = ifft(CS_RecSpectrum,[],1);
        CS_RecFID = CS_RecFID * normalizedFactor;
        CS_RecFID = ifft(CS_RecFID,[],2);
        CS_RLNE(noiseLevelID, lambdaID) = rlne(FID,CS_RecFID);
        
        % LRBHM-HTF
        LRBHM_Params.lambda = lambdaVec(1,lambdaID);
        LRBHM_RecHybridData = Solver_HybridSignalReconstruction_LRBHM_HTF(zerosFillingSampledSignal,mask,LRBHM_Params);
        
        LRBHM_RecHybridData = LRBHM_RecHybridData * normalizedFactor;
        LRBHM_RecFID = ifft(LRBHM_RecHybridData,[],2);
        all_LRBHM_RecFID(noiseLevelID,lambdaID,:,:) = LRBHM_RecFID;
        LRBHM_RLNE(noiseLevelID, lambdaID) = rlne(FID,LRBHM_RecFID);
    end
    
end

%% display RLNE vs lambda
figure, 
plot(log10(lambdaVec),LRBHM_RLNE(1,:)); hold on;
plot(log10(lambdaVec),LRBHM_RLNE(2,:)); hold on;
plot(log10(lambdaVec),LRBHM_RLNE(3,:)); hold on;
plot(log10(lambdaVec),LRBHM_RLNE(4,:));

%% display spectra

figure,mesh(abs(fft(hybridData,[],1)));
figure,mesh(abs(fft2(squeeze(all_LRBHM_RecFID(3,2,:,:)))));
figure,mesh(abs(fft2(squeeze(all_LRBHM_RecFID(3,6,:,:)))));
figure,mesh(abs(fft2(squeeze(all_LRBHM_RecFID(3,15,:,:)))));

%% ----------------- The end of the file ----------------------------------