%% Reproduce results of Figure 3 (missing data recovery in the spectrum) in the follwing paper:
%% Hengfa Lu et al., Low rank enhanced matrix recovery of hybrid time and frequency data in fast magnetic resonance spectroscopy, IEEE Transactions on Biomedical Engineering, 10.1109/TBME.2017.2719709, 2017.
%% Created by Hengfa Lu
%% Corresponding Author: Xiaobo Qu
%% Homepage: http://csrc.xmu.edu.cn
%% Email: quxiaobo <|at|> xmu.edu.cn
%% Affiliations: Computational Sensing Group (CSG), Departments of Electronic Science, Xiamen University, Xiamen 361005, China
%% All rights are reserved by CSG.
clear all;
close all;
clc;
%% add path
currentFolder = pwd;
addpath(genpath(currentFolder));
%% generate 1D fid
numPoints = 512;
numPeaks = 5;
phase = zeros(numPeaks,1);
amp = ones(numPeaks,1);                  % amplitude
omga = [0.1 0.3 0.5 0.7 0.9];              % frequencies
tao = 16+64*[0.1 0.6 1.1 1.6 2.1];
[fid,t] = genrateFID_1D_Peaks(amp,omga,phase,tao,numPoints);    % [FID,t]=genrateFID_1D_Peaks(Amp,Omga,Phase,Tao,NumPoints)
FID = fid';
FID = FID / max( abs(FID) );
%% generate 1D spectrum
originalSpectrum = fft(FID);
%% sampling
samplingRate = 0.05;
mask = Generate_1D_SamplingMask(numPoints,samplingRate);
mask([51 52 53 154 155 156 256 257 258 358 359 360 461 462 463]) = 1; % The center frequency points are forced to be sampled. 
realSamplingRate = sum(mask(:))/numel(mask);
sampledSpec = Spectra_1D_DownSampling(originalSpectrum,mask);

%% LR-based method
params.tol = 1e-4;
params.lambda = 1e5;
params.hankel_k = round(0.5*size(originalSpectrum,1));

LR_truncation_spec_rec = Solver_SpectrumRecovery_LR(sampledSpec,mask,params);

%% sparsity-based method
CS_params.tol = 1e-4;
CS_params.lambda = 1e2;
CS_truncation_spec_rec = Solver_SpectrumRecovery_Sparsity(sampledSpec,mask,CS_params);

%% show reconstruction results
showLR_RecSpectra = LR_truncation_spec_rec./max(abs(LR_truncation_spec_rec(:)));
showCS_RecSpectra = CS_truncation_spec_rec./max(abs(CS_truncation_spec_rec(:)));
showSampledSpec = sampledSpec./max(abs(sampledSpec(:)));
normalizeOriginalSpectra = originalSpectrum./max(abs(originalSpectrum(:)));

figure,plot(abs(normalizeOriginalSpectra),'Color',[0.47 0.67 0.19],'LineWidth',3); hold on;
plot(find(mask==1),abs(normalizeOriginalSpectra(mask)),'.','Color',[0.85 0.33 0.1], 'MarkerSize', 25); xlim([1 512]); title('original spectrum and sampled points');
figure,
plot(abs(showLR_RecSpectra),'Color',[0 0.45 0.74],'LineWidth',3);xlim([1 512]); title('LR');
figure
plot(abs(showCS_RecSpectra),'Color',[0 0.45 0.74],'LineWidth',3);xlim([1 512]); title('CS');

%% -------------------- the end of the file ---------------------
