%% Reproduce Figures 5 and 7 (STEU COSY reconstruction) in the following paper:
%% Hengfa Lu et al., Low rank enhanced matrix recovery of hybrid time and frequency data in fast magnetic resonance spectroscopy, IEEE Transactions on Biomedical Engineering, 10.1109/TBME.2017.2719709, 2017.
%% Created by Hengfa Lu
%% Corresponding Author: Xiaobo Qu
%% Homepage: http://csrc.xmu.edu.cn
%% Email: quxiaobo <|at|> xmu.edu.cn
%% Affiliations: Computational Sensing Group (CSG), Department of Electronic Science, Xiamen University, Xiamen 361005, China
%% All rights are reserved by CSG. 
clc;
clear all;
close all;
%% record time and create log file
% programStartTime = datestr(now);
% diary( strcat('./log/SECOSY_LRBHM_vs_CS_PoissonGapSampling_SR_020_',programStartTime,'.txt') );
% diary on;
%% add path
currentFolder = pwd;
addpath(genpath(currentFolder));

%% load data and mask
load marrow_cosy
hybridData = dataO; 

[signal_row,signal_col] = size(hybridData);
normalizedFactor = max(abs(hybridData(:)));
hybridData = hybridData/normalizedFactor;

%% perform non-uniform sampling
sampingRatio = 0.20;
mask = Generate_2D_HTFPlanePoissonMask(sampingRatio,signal_row,signal_col);
mask = flipud(mask);    % This is important! The sampling patterm should fit the signal.
zerosFillingSampledSignal = zeros(signal_row,signal_col);
index = find(mask==1);
zerosFillingSampledSignal(index) = hybridData(index);

%% Block Hankel reconstruction
LRBHM_params.tol = 5*1e-3;
LRBHM_params.lambda = 1e5;

t_start = clock;
LRBHM_RecHybridData = Solver_HybridSignalReconstruction_LRBHM_HTF(zerosFillingSampledSignal,mask,LRBHM_params);
t_end = clock;
LRBHM_RunTime = etime(t_end,t_start);

LRBHM_RecSpectrum = fft(LRBHM_RecHybridData,[],1);

%% Compressed Sensing(CS) reconstruction with sparsity constraint
CS_params.lambda = 1e5;
CS_params.tol = 1e-3;

t_start = clock;

CS_RecSpectrum = Solver_HybridSignalReconstruction_CS(zerosFillingSampledSignal,mask,CS_params);

t_end = clock;
CS_RunTime = etime(t_end,t_start);

%% show reconstruction spectra
N1 = signal_col;
N2 = 100;
fn1 = 1024;
fn2 = 1024;
CTlevel = 90;
win1Weight = 0.4;
win2Weight = 0.6;

SpectrumPostProcessing(ifft(circshift(fft(hybridData,[],1),[-2,5]),[],1),N1,N2,fn1,fn2,win1Weight,win2Weight,1,CTlevel); title('Reference');
SpectrumPostProcessing(ifft(circshift(CS_RecSpectrum,[-2,5]),[],1),N1,N2,fn1,fn2,win1Weight,win2Weight,1,CTlevel); title('CS');
SpectrumPostProcessing(ifft(circshift(LRBHM_RecSpectrum,[-2,5]),[],1),N1,N2,fn1,fn2,win1Weight,win2Weight,1,CTlevel); title('LRBHM-HTF');

%% perform peak intensities correlation analysis between ground-truth spectrum and reconstructed spectrum.
N1 = 100;
N2 = 100;
fn1 = 1024; % This parameter should not be changed unless the peaks coordinates change together.
fn2 = 1024; % This parameter should not be changed unless the peaks coordinates change together.
CTlevel = 90;

abs_OriginalSpectrum = SpectrumPostProcessing( hybridData,N1,N2,fn1,fn2,win1Weight,win2Weight,0,CTlevel);
abs_CS_RecSpectrum = SpectrumPostProcessing( ifft(CS_RecSpectrum,[],1),N1,N2,fn1,fn2,win1Weight,win2Weight,0,CTlevel);
abs_LRBHM_RecSpectrum = SpectrumPostProcessing( ifft(LRBHM_RecSpectrum,[],1),N1,N2,fn1,fn2,win1Weight,win2Weight,0,CTlevel);

% peaks coordinates (The  coordinates are determined manually.)
peaksCorr_COSY(abs_OriginalSpectrum,abs_CS_RecSpectrum,[0 0.25]);
peaksCorr_COSY(abs_OriginalSpectrum,abs_LRBHM_RecSpectrum,[0 0.25]);

peaksCorr_COSY(abs_OriginalSpectrum,abs_CS_RecSpectrum,[0 1]);
peaksCorr_COSY(abs_OriginalSpectrum,abs_LRBHM_RecSpectrum,[0 1]);

%% record end
% programEndTime = datestr(now);
% diary off;
%% ----------------- The end of the file ----------------------------------